Package: timechange
Title: Efficient Manipulation of Date-Times
Version: 0.4.0
Authors@R: c(person("Vitalie", "Spinu", email = "spinuvit@gmail.com", role = c("aut", "cre")),
             person("Google Inc.", role = c("ctb", "cph")))
Description: Efficient routines for manipulation of date-time objects while
   accounting for time-zones and daylight saving times. The package includes
   utilities for updating of date-time components (year, month, day etc.),
   modification of time-zones, rounding of date-times, period addition and
   subtraction etc. Parts of the 'CCTZ' source code, released under the Apache
   2.0 License, are included in this package. See
   <https://github.com/google/cctz> for more details.
Depends: R (>= 3.3)
License: GPL (>= 3)
Encoding: UTF-8
LinkingTo: cpp11 (>= 0.2.7)
Suggests: testthat (>= 0.7.1.99), knitr
SystemRequirements: A system with zoneinfo data (e.g.
        /usr/share/zoneinfo). On Windows the zoneinfo included with R
        is used.
BugReports: https://github.com/vspinu/timechange/issues
URL: https://github.com/vspinu/timechange/
RoxygenNote: 7.2.1
NeedsCompilation: yes
Packaged: 2026-01-29 08:59:34 UTC; vitalie
Author: Vitalie Spinu [aut, cre],
  Google Inc. [ctb, cph]
Maintainer: Vitalie Spinu <spinuvit@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-29 22:20:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:44:04 UTC; unix
Archs: timechange.so.dSYM
