# Generated by cpp11: do not edit by hand

C_time_get <- function(dt, components, week_start) {
  .Call(`_timechange_C_time_get`, dt, components, week_start)
}

C_time_ceiling <- function(dt, unit_name, nunits, week_start, change_on_boundary, origin) {
  .Call(`_timechange_C_time_ceiling`, dt, unit_name, nunits, week_start, change_on_boundary, origin)
}

C_time_floor <- function(dt, unit_name, nunits, week_start, origin) {
  .Call(`_timechange_C_time_floor`, dt, unit_name, nunits, week_start, origin)
}

C_local_tz <- function() {
  .Call(`_timechange_C_local_tz`)
}

C_valid_tz <- function(tz_name) {
  .Call(`_timechange_C_valid_tz`, tz_name)
}

C_time_update <- function(dt, updates, tz, roll_month, roll_dst, week_start, exact) {
  .Call(`_timechange_C_time_update`, dt, updates, tz, roll_month, roll_dst, week_start, exact)
}

C_time_add <- function(dt, periods, roll_month, roll_dst) {
  .Call(`_timechange_C_time_add`, dt, periods, roll_month, roll_dst)
}

C_force_tz <- function(dt, tz, roll_dst) {
  .Call(`_timechange_C_force_tz`, dt, tz, roll_dst)
}

C_force_tzs <- function(dt, tzs, tz_out, roll_dst) {
  .Call(`_timechange_C_force_tzs`, dt, tzs, tz_out, roll_dst)
}

C_local_clock <- function(dt, tzs) {
  .Call(`_timechange_C_local_clock`, dt, tzs)
}
